<?php

// Check if class already exists

//if (!class_exists("WPsite_Flyout_Fire_List_Table")):

/**
 * Include the WP_List_Table library
 *
 * @since 1.0.0
 *
 */
if(!class_exists('WP_List_Table')){
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

/**
 * This class extends WP_List_Table and is used to create the table as
 *	seen in the admin panel.
 *
 * @since 1.0.0
 *
 * @extends	WP_List_Table
 */
class WPsite_Flyout_Fire_List_Table extends WP_List_Table {

	/**
	 * Construtor
	 *
	 * @since 1.0.0
	 *
	 * @param	N/A
	 * @return	Instance
	 */
	function __construct(){
        global $status, $page;

        //Set parent defaults
        parent::__construct( array(
            'singular'  => 'flyout fire',     //singular name of the listed records
            'plural'    => 'flyout fires',    //plural name of the listed records
            'ajax'      => false        //does this table support ajax?
        ) );
    }

	/**
	 * Called if there are no flyout_fires
	 *
	 * @since 1.0.0
	 *
	 * @param	N/A
	 * @return	N/A
	 */
	function no_items() {
		_e('No flyout fires found.', WPsite_Flyout_Fire::$text_domain);
	}

	/**
	 * Called for any colunm without a realted function
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @param	array $column_name The name/slug of the column to be processed
	 * @return	string Text or HTML to be placed inside the column <td>
	 */
	function column_default( $item, $column_name ) {

		$args = unserialize($item['args']);

		switch( $column_name ) {
			case 'status':
				return $item['active'];
			default:
				return '<label>' . $item[$column_name] . '</label>';
		}
	}

	/**
	 * Custom function for the name column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_display_on($item) {

        $trigger = unserialize($item['trigger']);

        $display_on = '';

        switch ($trigger['type']) {
	        case 'site':
	        	$display_on = 'Site Wide';
	        break;

	        case 'frontpage':
	        	$display_on = 'Front Page';
	        break;

	        case 'tag':

	        	$display_on_tag = '<ul style="padding-left: 20px;list-style-type: disc;">';

	        	$tags = get_tags(array('include' => $trigger['tag']['id']));

	        	foreach ($tags as $tag) {
		        	$display_on_tag .= '<li><a href="' . get_tag_link($tag->term_id) . '" target="_blank">' . $tag->name . '</a></li>';
	        	}

	        	$display_on_tag .= '</ul>';

	        	$display_on .= '<a role="button" type="button" data-toggle="modal" data-target="#' . $item['id'] . '-' . $trigger['type'] . '">' . __('Certain ', WPsite_Flyout_Fire::$text_domain) . ucfirst($trigger['type']) . 's</a><div class="modal fade" id="' . $item['id'] . '-' . $trigger['type'] . '" tabindex="-1" role="dialog"><div class="modal-dialog" style="width: 400px;"><div class="modal-content"><div class="modal-header"><button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button><h4 class="modal-title">' . ucfirst($trigger['type']) . 's</h4></div><div class="modal-body">' . $display_on_tag . '</div><div class="modal-footer"><button type="button" class="btn btn-default" data-dismiss="modal">Close</button></div></div></div></div>';

	        break;

	        case 'category':

	        	if ( isset($trigger['category']['all']) && $trigger['category']['all'] ) {
					$display_on .= 'Category Archives';
	        	} else {

		        	$display_on_category = '<ul style="padding-left: 20px;list-style-type: disc;">';

		        	$categories = get_categories(array('include' => $trigger['category']['id']));

		        	foreach ($categories as $category) {
			        	$display_on_category .= '<li><a href="' . get_category_link($category->term_id) . '" target="_blank">' . $category->name . '</a></li>';
		        	}

		        	$display_on_category .= '</ul>';

		        	$display_on .= '<a role="button" type="button" data-toggle="modal" data-target="#' . $item['id'] . '-' . $trigger['type'] . '">' . __('Certain ', WPsite_Flyout_Fire::$text_domain) . ucfirst($trigger['type']) . 's</a><div class="modal fade" id="' . $item['id'] . '-' . $trigger['type'] . '" tabindex="-1" role="dialog"><div class="modal-dialog" style="width: 400px;"><div class="modal-content"><div class="modal-header"><button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button><h4 class="modal-title">' . ucfirst($trigger['type']) . 's</h4></div><div class="modal-body">' . $display_on_category . '</div><div class="modal-footer"><button type="button" class="btn btn-default" data-dismiss="modal">Close</button></div></div></div></div>';
	        	}

	        break;

	        case 'archive':

				$display_on .= 'Archive Pages';

	        break;

	        default:

				if ($trigger[$trigger['type']]['all']) {
					$display_on .= __('All', WPsite_Flyout_Fire::$text_domain) . ' ' . ucfirst($trigger['type'] . 's');
				} else {

					$display_on_post = '<ul style="padding-left: 20px;list-style-type: disc;">';

					if ($trigger[$trigger['type']]['id'] != '') {
						$posts = get_posts(array(
							'post_type'	=> $trigger['type'],
							'include' 	=> $trigger[$trigger['type']]['id']
						));

			        	foreach ($posts as $post) {
				        	$display_on_post .= '<li><a href="' . get_permalink($post->ID) . '" target="_blank">' . $post->post_title . '</a></li>';
			        	}
					}

					$display_on_post .= '</ul>';

 					$display_on .= '<a role="button" type="button" data-toggle="modal" data-target="#' . $item['id'] . '-' . $trigger['type'] . '">' . __('Certain ', WPsite_Flyout_Fire::$text_domain) . ucfirst($trigger['type']) . 's</a><div class="modal fade" id="' . $item['id'] . '-' . $trigger['type'] . '" tabindex="-1" role="dialog"><div class="modal-dialog" style="width: 400px;"><div class="modal-content"><div class="modal-header"><button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button><h4 class="modal-title">' . ucfirst($trigger['type']) . 's</h4></div><div class="modal-body">' . $display_on_post . '</div><div class="modal-footer"><button type="button" class="btn btn-default" data-dismiss="modal">Close</button></div></div></div></div>';

				}

	        break;
        }

        return rtrim($display_on, '<br/>');
	}

	/**
	 * Custom function for the name column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_color($item) {
		return '<div style="background-color:' . $item['color'] . '; color:' . $item['text_color'] . '; width:100%; height:30px;padding: 5px;" class="img-rounded">Example Text</div>';
	}

	/**
	 * Custom function for the name column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_name($item) {

		// Active

		if ($item['active'] == 'Active') {

			$actions['edit_1'] = sprintf('<a href="?page=%s&action=%s&set_action=%s&flyout_fire_id=%s&wp_nonce=%s" title="' . __('Deactivate', WPsite_Flyout_Fire::$text_domain) . '" class="' . WPsite_Flyout_Fire::$prefix_dash  . 'row-action ' . WPsite_Flyout_Fire::$prefix_dash  . 'inactivate"><i class="fa fa-power-off"></i></a>',
				$_REQUEST['page'],
				'active',
				'Inactive',
				$item['id'],
				wp_create_nonce('wpsite_flyout_fire_activate'));
		}

		// Inactive

		if ($item['active'] == 'Inactive') {

			$actions['edit_1'] = sprintf('<a href="?page=%s&action=%s&set_action=%s&flyout_fire_id=%s&wp_nonce=%s" title="' . __('Activate', WPsite_Flyout_Fire::$text_domain) . '" class="' . WPsite_Flyout_Fire::$prefix_dash  . 'row-action ' . WPsite_Flyout_Fire::$prefix_dash  . 'activate"><i class="fa fa-power-off"></i></a>',
				$_REQUEST['page'],
				'active',
				'Active',
				$item['id'],
				wp_create_nonce('wpsite_flyout_fire_activate'));
		}

		// Edit

        $actions['edit_2'] = sprintf('<a href="?page=%s&action=edit&flyout_fire_id=%s&wp_nonce=%s" title="' . __('Edit', WPsite_Flyout_Fire::$text_domain) . '" class="' . WPsite_Flyout_Fire::$prefix_dash  . 'row-action">%s</a>',
        	WPsite_Flyout_Fire::$add_new_page,
        	$item['id'],
        	wp_create_nonce('wpsite_flyout_fire_edit'),
        	'<i class="fa fa-cogs"></i>');

		// Preview

		$actions['edit_3'] = sprintf('<a href="?page=%s&action=%s&flyout_fire_id=%s&wp_nonce=%s" title="' . __('Preview', WPsite_Flyout_Fire::$text_domain) . '" class="' . WPsite_Flyout_Fire::$prefix_dash  . 'row-action">%s</a>',
			$_REQUEST['page'],
			'preview',
			$item['id'],
			wp_create_nonce('wpsite_flyout_fire_preview'),
			'<i class="fa fa-eye"></i>');

		// Stats

        $actions['edit_4'] = sprintf('<a href="?page=%s&flyout_fire_id=%s&flyout_fire_name=%s" title="' . __('Stats', WPsite_Flyout_Fire::$text_domain) . '" class="' . WPsite_Flyout_Fire::$prefix_dash  . 'row-action">%s</a>',
        	WPsite_Flyout_Fire::$stats_page,
        	$item['id'],
        	$item['name'],
        	'<i class="fa fa-bar-chart"></i>');

		// Delete

        $actions['delete'] = sprintf('<a href="#" title="' . __('Delete', WPsite_Flyout_Fire::$text_domain) . '" class="' . WPsite_Flyout_Fire::$prefix_dash . 'delete ' . WPsite_Flyout_Fire::$prefix_dash  . 'row-action" data-toggle="modal" data-target="#myModal" data-id="%s" data-url="?page=%s&action=delete&flyout_fire_id=%s&wp_nonce=%s">%s</a>',
        	$item['name'],
        	WPsite_Flyout_Fire::$dashboard_page,
        	$item['id'],
        	wp_create_nonce('wpsite_flyout_fire_delete'),
        	'<i class="fa fa-trash-o"></i>');

        // Return the title contents

        return sprintf('%1$s <small style="opacity: 0.5;">id:(%2$s)</small> %3$s',
            $item['name'],
            $item['id'],
            $this->row_actions($actions)
        );
	}


	/**
	 * Custom function for the status column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_status($item) {

		if ($item['active'] == 'Active') {
			$name = '<label style="color:green;"><span class="glyphicon glyphicon-ok"></span> ON</label>';
			$actions = array(
	            'Deactivate'	=> sprintf('<a href="?page=%s&action=%s&set_action=%s&flyout_fire_id=%s&wp_nonce=%s" class="btn btn-danger btn-xs"><span style="color:white;" class="glyphicon glyphicon-off"></span></a>',$_REQUEST['page'],'active','Inactive',$item['id'],wp_create_nonce('wpsite_flyout_fire_activate'))
	        );
		}

		if ($item['active'] == 'Inactive') {
			$name = '<label style="color:red;"><span class="glyphicon glyphicon-minus"></span> OFF</label>';
			$actions = array(
	            'Activate'	=> sprintf('<a href="?page=%s&action=%s&set_action=%s&flyout_fire_id=%s&wp_nonce=%s" class="btn btn-success btn-xs"><span style="color:white;" class="glyphicon glyphicon-off"></span></a>',$_REQUEST['page'],'active','Active',$item['id'],wp_create_nonce('wpsite_flyout_fire_activate'))
	        );
		}

        return sprintf('%1$s %2$s',
            $name,
            $this->row_actions($actions)
        );
	}

	/**
	 * Custom function for the impressions column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_impressions($item) {

		$stats = WPsite_Stats_FF::getStats(null, null, $item['id']);

		$impressions_total = 0;
		foreach ($stats as $stat) {
		    $impressions_total += $stat['impressions'];
		}

		return number_format($impressions_total);
	}

	/**
	 * Custom function for the clicks column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_clicks($item) {

		$stats = WPsite_Stats_FF::getStats(null, null, $item['id']);

		$clicks_total = 0;
		foreach ($stats as $stat) {
		    $clicks_total += $stat['clicks'];
		}

		return number_format($clicks_total);
	}

	/**
	 * Custom function for the clicks column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_click_through_rate($item) {

		$stats = WPsite_Stats_FF::getStats(null, null, $item['id']);

		$impressions_total = 0;
		$clicks_total = 0;
		foreach ($stats as $stat) {
		    $clicks_total += $stat['clicks'];
		    $impressions_total += $stat['impressions'];
		}

        $ctr = $impressions_total != 0 ? round(($clicks_total/$impressions_total) * 100, 2) : 0;

        //Return the title contents

        return $ctr . '%';
	}

	/**
	 * Custom function for the impressions column
	 *
	 * @since 1.0.0
	 *
	 * @param	array $item A singular item (one full row's worth of data)
	 * @return	string Text to be placed inside the column <td> (movie title only)
	 */
	function column_date_range($item) {

		//$item['args'] = unserialize($item['args']);

		return date('M d, Y', strtotime($item['args']['start_date'])) . ' - ' . date('M d, Y', strtotime($item['args']['end_date']));
	}

	/**
	 * This method dictates the table's columns and titles.
	 *
	 * @since 1.0.0
	 *
	 * @param	N/A
	 * @return	array An associative array containing column information: 'slugs'=>'Visible Titles'
	 */
	function get_columns(){
		$columns = array(
			//'status' 				=> __( 'Status', WPsite_Flyout_Fire::$text_domain),
			'name' 					=> __( 'Name', WPsite_Flyout_Fire::$text_domain),
			'display_on' 			=> __( 'Display On', WPsite_Flyout_Fire::$text_domain),
			'impressions'        	=> __( 'Impressions', WPsite_Flyout_Fire::$text_domain),
			'clicks'             	=> __( 'Clicks', WPsite_Flyout_Fire::$text_domain),
			'click_through_rate' 	=> __( 'Click-Through Rate', WPsite_Flyout_Fire::$text_domain),
			'date_range' 			=> __( 'Date Range', WPsite_Flyout_Fire::$text_domain),
			//'color' 				=> __( 'Color', WPsite_Flyout_Fire::$text_domain)
		);
		return $columns;
	}

	/**
	 * Prepares data for display.
	 *
	 * @since 1.0.0
	 *
	 * @param	N/A
	 * @return	N/A
	 */
	function prepare_items() {
		global $wpdb;

        $per_page = 50;

        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();

        $this->_column_headers = array($columns, $hidden, $sortable);

        //$this->process_bulk_action();

        $orderby = (!empty($_REQUEST['orderby'])) ? $_REQUEST['orderby'] : 'id';
        $order = (!empty($_REQUEST['order'])) ? $_REQUEST['order'] : 'asc';

        $active = '';

        if ( isset($_GET['status']) && $_GET['status'] == 'active' ) {
	        $active = "WHERE `active` = 'Active'";
        }

        else if ( isset($_GET['status']) && $_GET['status'] == 'inactive' ) {
	        $active = "WHERE `active` = 'Inactive'";
        }

		$data = $wpdb->get_results($wpdb->prepare("SELECT * FROM `" . WPsite_Flyout_Fire_DB::getTableName() . "` " . $active . " ORDER BY %s %s;", $orderby, $order ),'ARRAY_A');

        $current_page = $this->get_pagenum();

        $total_items = count($data);


        /**
         * The WP_List_Table class does not handle pagination for us, so we need
         * to ensure that the data is trimmed to only the current page. We can use
         * array_slice() to
         */
        $data = array_slice($data,(($current_page-1)*$per_page),$per_page);

        $this->items = $data;

        $this->set_pagination_args( array(
            'total_items' => $total_items,                  //WE have to calculate the total number of items
            'per_page'    => $per_page,                     //WE have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)   //WE have to calculate the total number of pages
        ) );
	}

	/**
	 * Get a list of CSS classes for the <table> tag
	 *
	 * @since 3.1.0
	 * @access protected
	 *
	 * @return array
	 */
	public function get_table_classes() {
	    return array('table table-responsive');
	}

	/**
	 * Display the table
	 *
	 * @since 3.1.0
	 * @access public
	 */
	public function display() {
		$singular = $this->_args['singular'];

		//$this->display_tablenav( 'top' );

		?>
		<table class="<?php echo implode( ' ', $this->get_table_classes() ); ?>">
			<thead>
			<tr>
				<?php $this->print_column_headers(); ?>
			</tr>
			</thead>

<!--
			<tfoot>
			<tr>
				<?php $this->print_column_headers( false ); ?>
			</tr>
			</tfoot>
-->

			<tbody id="the-list"<?php
				if ( $singular ) {
					echo " data-wp-lists='list:$singular'";
				} ?>>
				<?php $this->display_rows_or_placeholder(); ?>
			</tbody>
		</table>
		<?php
		$this->display_tablenav( 'bottom' );
	}

	/**
	 * Generates content for a single row of the table
     *
     * @since 3.1.0
     * @access public
     *
     * @param object $item The current item
     */

    public function single_row( $item ) {

	    // Check if this is a variant

		$item['args'] = WPsite_Flyout_Fire_DB::mb_unserialize($item['args']);

		$class = '';

		// Active

		if ( $item['active'] == 'Active' && ( !isset($item['args']['ab_testing']['active']) || ( isset($item['args']['ab_testing']['active']) && !$item['args']['ab_testing']['active'] ) )) {

			$class = WPsite_Flyout_Fire::$prefix_dash . 'active-row';

		}

		// Test

		else if ( $item['active'] == 'Active' && isset($item['args']['ab_testing']['active']) && $item['args']['ab_testing']['active'] ) {

			$class = WPsite_Flyout_Fire::$prefix_dash . 'test-row';

		}

		// Inactive

		else {

			$class = WPsite_Flyout_Fire::$prefix_dash . 'inactive-row';

		}

	    echo '<tr class="' . $class . '">';
		$this->single_row_columns( $item );
		echo '</tr>';

	}

	/**
	 * Generate row actions div
	 *
	 * @since 3.1.0
	 * @access protected
	 *
	 * @param array $actions The list of actions
	 * @param bool $always_visible Whether the actions should be always visible
	 * @return string
	 */
	protected function row_actions( $actions, $always_visible = false ) {

		$action_count = count( $actions );
		$i = 0;

		if ( !$action_count )
			return '';

		$out = '<div class="' . ( $always_visible ? 'row-actions visible' : 'row-actions' ) . '">';
		foreach ( $actions as $action => $link ) {
			++$i;
			( $i == $action_count ) ? $sep = '' : $sep = ' | ';
			$out .= "<span class='$action'>$link$sep</span>";
		}
		$out .= '</div>';

		return $out;
	}

	/**
	 * Generates and display row actions links for the list table.
	 *
	 * @since 4.3.0
	 * @access protected
	 *
	 * @param object $item        The item being acted upon.
	 * @param string $column_name Current column name.
	 * @param string $primary     Primary column name.
	 * @return string The row actions output. In this case, an empty string.
	 */
	protected function handle_row_actions( $item, $column_name, $primary ) {
		return '';
 	}
}

//endif;
?>